#include "BLKH.h"

/*
 * This file contains the main function of the program.
 */

int main(int argc, char *argv[])
{
    int Exp, Experiments = 10;
    GainType Bottleneck, BestBottleneck, BottleneckSum = 0;
    int Bound, LowerBound, Opt = 0;
    double Time, LastTime, LBTime;

    /* Read the specification of the problem */
    if (argc >= 2) {
        ParameterFileName = argv[1];
        if (argc >= 3)
            Experiments = atoi(argv[2]);
    }
    ReadParameters();
    ReadProblem();
    LastTime = GetTime();
    LowerBound = TwoMax();
    if ((Bound = BBSSP(LowerBound)) > LowerBound)
        LowerBound = Bound;
    if (ProblemType == ATSP) {
        if ((Bound = BBSSPA(LowerBound)) > LowerBound)
             LowerBound = Bound;
        if ((Bound = BSCSSP(LowerBound)) > LowerBound)
            LowerBound = Bound;
        if ((Bound = BAP(LowerBound)) > LowerBound)
            LowerBound = Bound;
    }
    LBTime = Time = fabs(GetTime() - LastTime);
    printff("Lower bound = %d, Time = %0.2fs\n", LowerBound, Time);
    assert(BestTour = (int *) malloc((DimensionSaved + 1) * sizeof(int)));
    BestBottleneck = PLUS_INFINITY;
    for (Exp = 1; Exp <= Experiments; Exp++, Seed++) {
        Bottleneck = SolveBTSP(LowerBound);
        if (OutputTourFileName) {
            int OldTraceLevel = TraceLevel;
            TraceLevel = 1;
            WriteTour(OutputTourFileName, BestTour, BestCost);
            TraceLevel = OldTraceLevel;
        }
        if (TourFileName &&
            (Optimum == MINUS_INFINITY || Bottleneck < BestBottleneck)) {
            int OldTraceLevel = TraceLevel;
            TraceLevel = 1;
            WriteTour(TourFileName, BestTour, BestCost);
            TraceLevel = OldTraceLevel;
        }
        if (Bottleneck < BestBottleneck)
            BestBottleneck = Bottleneck;
        if (TraceLevel > 0)
            printff("* Performed %d out of %d experiments\n\n",
                    Exp, Experiments);
        BottleneckSum += Bottleneck;
        if (Bottleneck == Optimum)
            Opt++;
        else if (Bottleneck < Optimum) {
            Optimum = Bottleneck;
            Opt = 1;
        }
    }
    Time = GetTimeUsage(RUSAGE_SELF) + GetTimeUsage(RUSAGE_CHILDREN);
    Time += (Experiments - 1) * LBTime;
    if (Optimum != MINUS_INFINITY)
        printff
            ("Opt = " GainFormat ", Best = " GainFormat ", "
             "Avg. = %0.1f, Error = %0.2f%%\n"
             "Opt = %0.0f%%, Time = %0.2fs\n\n",
             Optimum, BestBottleneck, (double) BottleneckSum / Experiments,
             ((double) BottleneckSum / Experiments -
              Optimum) / Optimum * 100.0, Opt * 100.0 / Experiments,
             Time / Experiments);
    else
        printff
            ("Best = " GainFormat ", "
             "Avg. = %0.1f, Time = %0.2fs\n\n",
             BestBottleneck, (double) BottleneckSum / Experiments,
             Time / Experiments);
    return 0;
}
