#include "BLKH.h"

/*
 * This file contains the main function of the BLKH program.
 */

int main(int argc, char *argv[])
{
    int Bottleneck, Bound, LowerBound;
    double Time, LastTime;

    if (argc >= 2)
        ParameterFileName = argv[1];
    ReadParameters();
    ReadProblem();
    LastTime = GetTime();

    /* Compute lower bound */
    LowerBound = TwoMax();
    if ((Bound = BBSSP(LowerBound)) > LowerBound)
        LowerBound = Bound;
    if (ProblemType == ATSP) {
        if ((Bound = BBSSPA(LowerBound)) > LowerBound)
            LowerBound = Bound;
        if ((Bound = BSCSSP(LowerBound)) > LowerBound)
            LowerBound = Bound;
        if ((Bound = BAP(LowerBound)) > LowerBound)
            LowerBound = Bound;
    }
    Time = fabs(GetTime() - LastTime);
    printff("Lower bound = %d, Time = %0.2fs\n", LowerBound, Time);
    assert(BestTour = (int *) malloc((DimensionSaved + 1) * sizeof(int)));

    /* Solve the BTSP */
    Bottleneck = SolveBTSP(LowerBound);

    Time = fabs(GetTime() - LastTime);
    TraceLevel = 1;
    if (OutputTourFileName)
        WriteTour(OutputTourFileName, BestTour, BestCost);
    if (TourFileName
        && (Optimum == MINUS_INFINITY || Bottleneck < Optimum))
        WriteTour(TourFileName, BestTour, BestCost);
    printff("Value = %d", Bottleneck);
    if (Optimum != MINUS_INFINITY && Optimum != 0)
        printff(", Gap = %0.2f%%",
                100.0 * (Bottleneck - Optimum) / Optimum);
    printff(", Time = %0.2fs\n\n",
            GetTimeUsage(RUSAGE_SELF) + GetTimeUsage(RUSAGE_CHILDREN));
    return 0;
}
